unit log_appenders;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, log_unit;

type

  { TWritelnLogAppender }

  TWritelnLogAppender = class(TLogAppender, ILogAppender)
  public
    function appendMessage(const aMessage: PLogMessage): boolean;
  end;

  { TFileLogAppender }

  TFileLogAppender = class(TLogAppender, ILogAppender)
  public
    constructor Create(const aFilename: string);
  private
    fFilename: string;
    fStream: TFileStream;
    procedure startup;
  public
    property filename: string read fFilename;
    property stream: TFileStream read fStream;
    function appendMessage(const aMessage: PLogMessage): boolean;
    destructor Destroy; override;
  end;


implementation


{ TWritelnLogAppender }

function TWritelnLogAppender.appendMessage(const aMessage: PLogMessage): boolean;
var
  s: string;
begin
  result:=false;
  s:=TLog.defaultFormat(aMessage);
  WriteLN(s);
  result:=true;
end;


{ TFileLogAppender }

constructor TFileLogAppender.Create(const aFilename: string);
begin
  inherited Create;
  fFilename:=aFilename;
  startup;
end;

procedure TFileLogAppender.startup;
begin
  try
    fStream:=TFileStream.Create(filename, fmCreate  or fmOpenWrite);
  except
    on e: Exception do
    begin
      FreeAndNil(fStream);
      raise e;
    end;
  end;
end;

function TFileLogAppender.appendMessage(const aMessage: PLogMessage): boolean;
var
  s: string;
  p: PChar;
begin
  result:=false;
  if stream = nil then exit(false);
  s:=TLog.defaultFormat(aMessage);
  s += LineEnding;
  p:=PChar(s);
  stream.Write(p^, length(s));
  result:=true;
end;

destructor TFileLogAppender.Destroy;
begin
  stream.Free;
  inherited Destroy;
end;


end.

